@@ -13,7 +13,7 @@ from uuid import UUID, uuid4
 import boto3
 import yaml
 
-from taskcat._cfn.template import Template
+from taskcat._cfn.template import Template, tcat_template_cache
 from taskcat._common_utils import ordered_dump, pascal_to_snake, s3_url_maker
 from taskcat._dataclasses import Tag, TestRegion
 
@@ -28,7 +28,7 @@ GENERIC_ERROR_PATTERNS = [
 def criteria_matches(criteria: dict, instance):
     # fail if criteria includes an invalid property
     for k in criteria:
-        if k not in instance.__dict__:
+        if not hasattr(instance, k):
             raise ValueError(f"{k} is not a valid property of {type(instance)}")
     for k, v in criteria.items():
         # matching is AND for multiple criteria, so as soon as one fails,
@@ -56,6 +56,7 @@ class StackStatus:
         "UPDATE_ROLLBACK_FAILED",
         "UPDATE_ROLLBACK_COMPLETE_CLEANUP_IN_PROGRESS",
         "UPDATE_ROLLBACK_COMPLETE",
+        "OUT_OF_ORDER_EVENT",
     ]
 
 
@@ -206,6 +207,7 @@ class Stack:  # pylint: disable=too-many-instance-attributes
         self.region_name = region.name
         self.client: boto3.client = region.client("cloudformation")
         self.completion_time: timedelta = timedelta(0)
+        self.role_arn = region.role_arn
 
         # properties from additional cfn api calls
         self._events: Events = Events()
@@ -216,7 +218,7 @@ class Stack:  # pylint: disable=too-many-instance-attributes
         self.parameters: List[Parameter] = []
         self.creation_time: datetime = datetime.fromtimestamp(0)
         self.deletion_time: datetime = datetime.fromtimestamp(0)
-        self.status: str = ""
+        self._status: str = ""
         self.status_reason: str = ""
         self.disable_rollback: bool = False
         self.timeout_in_minutes: int = 0
@@ -225,6 +227,7 @@ class Stack:  # pylint: disable=too-many-instance-attributes
         self.tags: List[Tag] = []
         self.parent_id: str = ""
         self.root_id: str = ""
+        self._launch_succeeded: bool = False
         self._auto_refresh_interval: timedelta = timedelta(seconds=60)
         self._last_event_refresh: datetime = datetime.fromtimestamp(0)
         self._last_resource_refresh: datetime = datetime.fromtimestamp(0)
@@ -249,6 +252,34 @@ class Stack:  # pylint: disable=too-many-instance-attributes
             return True
         return False
 
+    @property
+    def status(self):
+        if self._status in StackStatus.COMPLETE:
+            if not self.launch_succeeded:
+                self._status = "OUT_OF_ORDER_EVENT"
+                self.status_reason = (
+                    "COMPLETE event not detected. "
+                    + "Potential out-of-band action against the stack."
+                )
+        return self._status
+
+    @status.setter
+    def status(self, status):
+        _complete = StackStatus.COMPLETE.copy()
+        del _complete[_complete.index("DELETE_COMPLETE")]
+        self._status = status
+        if status in StackStatus.FAILED:
+            self._launch_succeeded = False
+            return
+        if status in _complete:
+            self._launch_succeeded = True
+            return
+        return
+
+    @property
+    def launch_succeeded(self):
+        return self._launch_succeeded
+
     @classmethod
     def create(
         cls,
@@ -274,15 +305,19 @@ class Stack:  # pylint: disable=too-many-instance-attributes
                 region.client("s3"),
                 region.s3_bucket.auto_generated,
             ),
+            template_cache=tcat_template_cache,
         )
-        stack_id = cfn_client.create_stack(
-            StackName=stack_name,
-            TemplateURL=template.url,
-            Parameters=parameters,
-            DisableRollback=disable_rollback,
-            Tags=tags,
-            Capabilities=Capabilities.ALL,
-        )["StackId"]
+        create_options = {
+            "StackName": stack_name,
+            "TemplateURL": template.url,
+            "Parameters": parameters,
+            "DisableRollback": disable_rollback,
+            "Tags": tags,
+            "Capabilities": Capabilities.ALL,
+        }
+        if region.role_arn:
+            create_options["RoleARN"] = region.role_arn
+        stack_id = cfn_client.create_stack(**create_options)["StackId"]
         stack = cls(region, stack_id, template, test_name, uuid)
         # fetch property values from cfn
         stack.refresh()
@@ -296,18 +331,36 @@ class Stack:  # pylint: disable=too-many-instance-attributes
     def _import_child(  # pylint: disable=too-many-locals
         cls, stack_properties: dict, parent_stack: "Stack"
     ) -> Optional["Stack"]:
-        url = ""
-        for event in parent_stack.events():
-            if event.physical_id == stack_properties["StackId"] and event.properties:
-                url = event.properties["TemplateURL"]
-        if url.startswith(parent_stack.template.url_prefix()):
-            # Template is part of the project, discovering path
-            relative_path = url.replace(parent_stack.template.url_prefix(), "").lstrip(
-                "/"
-            )
-            absolute_path = parent_stack.template.project_root / relative_path
-        else:
-            try:
+        try:
+            url = ""
+            for event in parent_stack.events():
+                if (
+                    event.physical_id == stack_properties["StackId"]
+                    and event.properties
+                ):
+                    url = event.properties["TemplateURL"]
+            if url.startswith(parent_stack.template.url_prefix()):
+                # Template is part of the project, discovering path
+                relative_path = url.replace(
+                    parent_stack.template.url_prefix(), ""
+                ).lstrip("/")
+                absolute_path = parent_stack.template.project_root / relative_path
+                if not absolute_path.is_file():
+                    # try with the base folder stripped off
+                    relative_path2 = Path(relative_path)
+                    relative_path2 = relative_path2.relative_to(
+                        *relative_path2.parts[:1]
+                    )
+                    absolute_path = parent_stack.template.project_root / relative_path2
+                if not absolute_path.is_file():
+                    LOG.warning(
+                        f"Failed to find template for child stack "
+                        f"{stack_properties['StackId']}. tried "
+                        f"{parent_stack.template.project_root / relative_path}"
+                        f" and {absolute_path}"
+                    )
+                    return None
+            else:
                 # Assuming template is remote to project and downloading it
                 cfn_client = parent_stack.client
                 tempate_body = cfn_client.get_template(
@@ -326,28 +379,26 @@ class Stack:  # pylint: disable=too-many-instance-attributes
                 if not isinstance(tempate_body, str):
                     tempate_body = ordered_dump(tempate_body, dumper=yaml.SafeDumper)
                 if not absolute_path.exists():
-                    with open(absolute_path, "w") as fh:
+                    with open(absolute_path, "w", encoding="utf-8") as fh:
                         fh.write(tempate_body)
-            except Exception as e:  # pylint: disable=broad-except
-                LOG.warning(
-                    f"Failed to attach child stack "
-                    f'{stack_properties["StackId"]} {str(e)}'
-                )
-                LOG.debug("traceback", exc_info=True)
-                return None
-        template = Template(
-            template_path=str(absolute_path),
-            project_root=parent_stack.template.project_root,
-            url=url,
-        )
-        stack = cls(
-            parent_stack.region,
-            stack_properties["StackId"],
-            template,
-            parent_stack.name,
-            parent_stack.uuid,
-        )
-        stack.set_stack_properties(stack_properties)
+            template = Template(
+                template_path=str(absolute_path),
+                project_root=parent_stack.template.project_root,
+                url=url,
+                template_cache=tcat_template_cache,
+            )
+            stack = cls(
+                parent_stack.region,
+                stack_properties["StackId"],
+                template,
+                parent_stack.name,
+                parent_stack.uuid,
+            )
+            stack.set_stack_properties(stack_properties)
+        except Exception as e:  # pylint: disable=broad-except
+            LOG.warning(f"Failed to import child stack: {str(e)}")
+            LOG.debug("traceback:", exc_info=True)
+            return None
         return stack
 
     @classmethod
