@@ -41,7 +41,10 @@ class StackURLHelper:
     }
 
     def __init__(
-        self, template_mappings=None, template_parameters=None, parameter_values=None,
+        self,
+        template_mappings=None,
+        template_parameters=None,
+        parameter_values=None,
     ):
         if template_mappings:
             self.mappings = template_mappings
@@ -108,7 +111,7 @@ class StackURLHelper:
         # replace each key we have a value for
         for key in values:
             rep_text = "##" + key + "##"
-            rep_with = "" + values[key] + ""
+            rep_with = "" + str(values[key]) + ""
 
             result = result.replace(rep_text, rep_with)
 
@@ -117,7 +120,6 @@ class StackURLHelper:
     @staticmethod
     def values_to_dict(values):
         """Rewrite sub vars with actual variable values"""
-        LOG.debug("Rewrite Values: {}".format(values))
         # Create dictionary of values
         values_dict_string = values.replace("(", "{")
         values_dict_string = values_dict_string.replace(")", "}")
@@ -144,18 +146,14 @@ class StackURLHelper:
                         value, '"' + value + '"'
                     )
 
-        LOG.debug("Rewrite Values cleaned: {}".format(values_dict_string))
-
         values_dict = json.loads(values_dict_string)
 
         return values_dict
 
     def evaluate_fn_sub(self, expression):
-        """ Return expression with values replaced """
+        """Return expression with values replaced"""
         results = []
 
-        LOG.debug("Fn::Sub: '{}'".format(expression))
-
         # Builtins - Fudge some defaults here since we don't have runtime info
         # ${AWS::Region} ${AWS::AccountId}
         expression = self.rewrite_sub_vars_with_values(expression, self.SUBSTITUTION)
@@ -163,9 +161,7 @@ class StackURLHelper:
         # Handle Sub of form [ StringToSub, { "key" : "value", "key": "value" }]
         if "[" in expression:
             temp_expression = expression.split("[")[1].split(",")[0]
-            LOG.debug("Fn::Sub: (expression) {}".format(temp_expression))
             values = expression.split("[")[1].split("(")[1].split(")")[0]
-            LOG.debug("Fn::Sub: (values) {}".format(values))
             values = self.values_to_dict("(" + values + ")")
             temp_expression = self.rewrite_sub_vars_with_values(temp_expression, values)
         else:
@@ -175,13 +171,12 @@ class StackURLHelper:
         result = self.rewrite_sub_vars(temp_expression)
 
         results.append(result)
-        LOG.debug("Fn::Sub: '{}'".format(temp_expression))
 
         return results
 
     @staticmethod
     def evaluate_fn_join(expression):
-        """ Return the joined stuff """
+        """Return the joined stuff"""
         results = []
         new_values_list = []
 
@@ -202,15 +197,13 @@ class StackURLHelper:
 
     @staticmethod
     def evaluate_fn_if(expression):
-        """ Return both possible parts of the expression """
+        """Return both possible parts of the expression"""
         results = []
-        LOG.debug("Fn::If: {}".format(expression))
         value_true = expression.split(",")[1].strip()
         value_false = expression.split(",")[2].strip().strip("]")
         # if we don't have '' this can break things
         results.append("'" + value_true.strip("'") + "'")
         results.append("'" + value_false.strip("'") + "'")
-        LOG.debug("Fn::If: {}".format(results))
         return results
 
     def evaluate_fn_ref(self, expression):
@@ -219,12 +212,10 @@ class StackURLHelper:
         results = []
 
         temp = expression.split(": ")[1]
-        LOG.debug("Ref: {}".format(temp))
+        # pylint: disable=consider-iterating-dictionary
         if temp.strip("'") in self.SUBSTITUTION.keys():
-            LOG.debug("Ref: (found) {}".format(temp))
             temp = self.SUBSTITUTION[temp.strip("'")]
             temp = "'" + temp + "'"
-            LOG.debug("Ref: (found) {}".format(temp))
 
         results.append(temp)
 
@@ -311,13 +302,9 @@ class StackURLHelper:
 
             for replacement in replacements:
                 template_url_temp = template_url
-                LOG.debug("evaluate_string: (before) {}".format(template_url))
-                LOG.debug("expression: {}".format(parts[0]))
-                LOG.debug("replacement: {}".format(replacement))
                 template_url_temp = template_url_temp.replace(
                     "{" + parts[0] + "}", replacement
                 )
-                LOG.debug("evaluate_string: (after) {}".format(template_url_temp))
 
                 evaluated_strings = self.evaluate_string(
                     template_url_temp, depth=(depth + 1)
@@ -330,7 +317,7 @@ class StackURLHelper:
         return template_urls
 
     def _flatten_template_controller(self, template_url):
-        """ Recursively evaluate subs/ifs"""
+        """Recursively evaluate subs/ifs"""
         url_list = []
 
         # Replace ${SOMEVAR} with ##SOMEVAR## so finding actual "expressions" is easier
@@ -394,7 +381,6 @@ class StackURLHelper:
             final_template_path = Path(
                 "/".join([str(project_root), str(child_template_path_tmp)])
             )
-            LOG.debug("find_local_child_template: {}".format(final_template_path))
             if final_template_path.exists() and final_template_path.is_file():
                 return str(final_template_path)
 
@@ -410,14 +396,15 @@ class StackURLHelper:
             final_template_path = Path(
                 "/".join([str(project_root), str(child_template_path_tmp)])
             )
-            LOG.debug("find_local_child_template: {}".format(final_template_path))
             if final_template_path.exists() and final_template_path.is_file():
                 return str(final_template_path)
 
         return ""
 
     def template_url_to_path(
-        self, current_template_path, template_url,
+        self,
+        current_template_path,
+        template_url,
     ):
         child_local_paths = []
         child_template_paths = self.flatten_template_url(template_url)
