@@ -62,14 +62,8 @@ class Stacker:
         ]
         fan_out(self._create_stacks_for_test, {"tags": tags}, tests, threads)
 
-    def _get_stack_name_for_test(self, test):
-        prefix = f"{self.stack_name_prefix}-" if self.stack_name_prefix else ""
-        if self.shorten_stack_name:
-            return "{}{}-{}".format(prefix, test.name, self.uid.hex[:6])
-        return "{}{}-{}-{}".format(prefix, self.project_name, test.name, self.uid.hex)
-
     def _create_stacks_for_test(self, test, tags, threads: int = 32):
-        stack_name = self._get_stack_name_for_test(test)
+        stack_name = test.stack_name
         tags.append(Tag({"Key": "taskcat-project-name", "Value": self.project_name}))
         tags.append(Tag({"Key": "taskcat-test-name", "Value": test.name}))
         tags += test.tags
@@ -222,7 +216,11 @@ class Stacker:
                         project = v
                 if match and test and project:
                     stack = Stack.import_existing(
-                        stack_props, tests[test].template, region[0], test, uid
+                        stack_props,
+                        tests[test].template,
+                        region[0],
+                        test,
+                        uid,
                     )
                     stacks.append(stack)
         return stacks
@@ -235,7 +233,10 @@ class Stacker:
             if client not in stacks_by_client:
                 stacks_by_client[client] = {"Client": client, "Stacks": []}
             stacks_by_client[client]["Stacks"].append(stack)
-        return [stacks_by_client[r] for r in stacks_by_client]
+        return [
+            stacks_by_client[r]
+            for r in stacks_by_client  # pylint: disable=consider-using-dict-items
+        ]
 
     @staticmethod
     def list_stacks(profiles, regions):
@@ -286,5 +287,5 @@ class Stacker:
                 f"Failed to fetch stacks for region {region} using profile "
                 f"{profile} {type(e)} {e}"
             )
-            LOG.debug(f"Traceback:", exc_info=True)
+            LOG.debug("Traceback:", exc_info=True)
         return stacks
