@@ -3,8 +3,10 @@ import re
 import textwrap
 
 import cfnlint.core
+import cfnlint.helpers
 from cfnlint.config import ConfigMixIn as CfnLintConfig
 from jsonschema.exceptions import ValidationError
+from taskcat._common_utils import neglect_submodule_templates
 from taskcat._config import Config
 from taskcat._dataclasses import Templates
 
@@ -20,7 +22,7 @@ class Lint:
         Lints templates using cfn_python_lint. Uses config to define regions and
         templates to test. Recurses into child templates, excluding submodules.
 
-        :param config: path to tascat ci config file
+        :param config: path to taskcat ci config file
         """
         self._config: Config = config
         self._templates: Templates = templates
@@ -29,7 +31,17 @@ class Lint:
             self._cfnlint_config = CfnLintConfig([])
         except ValidationError as e:
             LOG.error("Error parsing cfn-lint config file: %s", str(e))
-        self._rules = cfnlint.core.get_rules([], [], [])
+            raise
+        self._rules = cfnlint.core.get_rules(
+            self._cfnlint_config.append_rules,
+            self._cfnlint_config.ignore_checks,
+            self._cfnlint_config.include_checks,
+            self._cfnlint_config.configure_rules,
+            self._cfnlint_config.include_experimental,
+            self._cfnlint_config.mandatory_checks,
+        )
+        if self._cfnlint_config.override_spec:
+            cfnlint.helpers.override_specs(self._cfnlint_config.override_spec)
         self.lints = self._lint()
         self.strict: bool = strict
 
@@ -60,12 +72,15 @@ class Lint:
             for template in self._templates.values():
                 templates.append(template)
                 templates += list(template.descendents)
+            templates = neglect_submodule_templates(
+                self._config.project_root, templates
+            )
             templates = set(templates)
             for template in templates:
                 self._run_checks(template, name, lint_errors, lints)
             for err in lint_errors:
                 LOG.error(err)
-        for test in lints:
+        for test in lints:  # pylint: disable=consider-using-dict-items
             for result in lints[test]["results"]:
                 if lints[test]["results"][result]:
                     if self._is_error(lints[test]["results"][result]):
@@ -90,25 +105,40 @@ class Lint:
             lint_errors.add(str(e))
         lints[name]["results"][tpath] = results
 
-    def output_results(self):
+    def output_results(self):  # noqa: C901
         """
         Prints lint results to terminal using taskcat console formatting
 
         :return:
         """
+        passed = set()
+        issue = {"warning": {}, "error": {}}
         lints = self.lints[0]
-        for test in lints:
+        for test in lints:  # pylint: disable=consider-using-dict-items
             for result in lints[test]["results"]:
                 if not lints[test]["results"][result]:
-                    LOG.info(f"Lint passed for test {test} on template {result}")
+                    passed.add(result)
                 else:
-                    msg = f"Lint detected issues for test {test} on template {result}:"
                     if self._is_error(lints[test]["results"][result]):
-                        LOG.error(msg)
+                        if result in issue["error"]:
+                            continue
+                        issue["error"][result] = lints[test]["results"][result]
                     else:
-                        LOG.warning(msg)
-                for inner_result in lints[test]["results"][result]:
-                    self._format_message(inner_result, test, result)
+                        if result in issue["warning"]:
+                            continue
+                        issue["warning"][result] = lints[test]["results"][result]
+        for filename in passed:
+            LOG.info(f"Linting passed for file: {filename}")
+        for filename, result_data in issue["warning"].items():
+            LOG.warning("---")
+            LOG.warning(f"Linting detected issues in: {filename}")
+            for result_message in result_data:
+                self._format_message(result_message, result_data)
+        for filename, result_data in issue["error"].items():
+            LOG.error("---")
+            LOG.error(f"Linting detected issues in: {filename}")
+            for result_message in result_data:
+                self._format_message(result_message, result_data)
 
     @property
     def passed(self):
@@ -128,7 +158,8 @@ class Lint:
                 return True
         return False
 
-    def _format_message(self, message, test, result):
+    @staticmethod
+    def _format_message(message, full_results):
         sev = message.rule.id[0]
         code = message.rule.id[1:]
         prefix = f"    line {message.linenumber} [{code}] [{message.rule.shortdesc}] "
@@ -138,9 +169,7 @@ class Lint:
         if sev == "E":
             LOG.error(message)
         elif sev == "W":
-            if "E" + code not in [
-                r.__str__().lstrip("[") for r in self.lints[0][test]["results"][result]
-            ]:
+            if "E" + code not in [r.__str__().lstrip("[") for r in full_results]:
                 LOG.warning(message)
         else:
             LOG.error("linter produced unkown output: " + message)
