@@ -1,4 +1,4 @@
-import collections
+import collections.abc
 import logging
 import os
 import re
@@ -8,11 +8,13 @@ from functools import reduce
 from pathlib import Path
 from time import sleep
 
-import boto3
+import requests
 import yaml
+from botocore.exceptions import ClientError
 
 from dulwich.config import ConfigFile, parse_submodules
 from taskcat.exceptions import TaskCatException
+from taskcat.regions_to_partitions import REGIONS
 
 LOG = logging.getLogger(__name__)
 
@@ -38,8 +40,21 @@ def s3_url_maker(bucket, key, s3_client, autobucket=False):
     retries = 10
     while True:
         try:
-            response = s3_client.get_bucket_location(Bucket=bucket)
-            location = response["LocationConstraint"]
+            try:
+                response = s3_client.get_bucket_location(Bucket=bucket)
+                location = response["LocationConstraint"]
+            except ClientError as e:
+                if e.response["Error"]["Code"] != "AccessDenied":
+                    raise
+                resp = requests.get(
+                    f"https://{bucket}.s3.amazonaws.com/{key}", timeout=3
+                )
+                location = resp.headers.get("x-amz-bucket-region")
+                if not location:
+                    # pylint: disable=raise-missing-from
+                    raise TaskCatException(
+                        f"failed to discover region for bucket {bucket}"
+                    )
             break
         except s3_client.exceptions.NoSuchBucket:
             if not autobucket or retries < 1:
@@ -55,12 +70,12 @@ def s3_url_maker(bucket, key, s3_client, autobucket=False):
     return url
 
 
-def get_s3_domain(region, ssm_client=None):
-    ssm_client = ssm_client if ssm_client else boto3.client("ssm")
-    partition = ssm_client.get_parameter(
-        Name=f"/aws/service/global-infrastructure/regions/{region}/partition"
-    )["Parameter"]["Value"]
-    return S3_PARTITION_MAP[partition]
+def get_s3_domain(region):
+    try:
+        return S3_PARTITION_MAP[REGIONS[region]]
+    except KeyError:
+        # pylint: disable=raise-missing-from
+        raise TaskCatException(f"cannot find the S3 hostname for region {region}")
 
 
 def s3_bucket_name_from_url(url):
@@ -109,16 +124,19 @@ def param_list_to_dict(original_keys):
     # - Used to give an Parameter => Index mapping for replacement.
     param_index = {}
     if not isinstance(original_keys, list):
+        # pylint: disable=raise-missing-from
         raise TaskCatException(
             'Invalid parameter file, outermost json element must be a list ("[]")'
         )
     for (idx, param_dict) in enumerate(original_keys):
         if not isinstance(param_dict, dict):
+            # pylint: disable=raise-missing-from
             raise TaskCatException(
                 'Invalid parameter %s parameters must be of type dict ("{}")'
                 % param_dict
             )
         if "ParameterKey" not in param_dict or "ParameterValue" not in param_dict:
+            # pylint: disable=raise-missing-from
             raise TaskCatException(
                 f"Invalid parameter {param_dict} all items must "
                 f"have both ParameterKey and ParameterValue keys"
@@ -142,7 +160,7 @@ def pascal_to_snake(pascal):
 
 def merge_nested_dict(old, new):
     for k, v in new.items():
-        if isinstance(old.get(k), dict) and isinstance(v, collections.Mapping):
+        if isinstance(old.get(k), dict) and isinstance(v, collections.abc.Mapping):
             merge_nested_dict(old[k], v)
         else:
             old[k] = v
@@ -199,3 +217,26 @@ def neglect_submodule_templates(project_root, template_list):
         if not gitmodule_template:
             finalized_templates.append(template_obj)
     return finalized_templates
+
+
+def determine_profile_for_region(auth_dict, region):
+    profile = auth_dict.get(region, auth_dict.get("default", "default"))
+    return profile
+
+
+def fetch_ssm_parameter_value(boto_client, parameter_path):
+    ssm = boto_client("ssm")
+    response = ssm.get_parameter(Name=parameter_path)
+    return response["Parameter"]["Value"]
+
+
+def fetch_secretsmanager_parameter_value(boto_client, secret_arn):
+    secrets_manager = boto_client("secretsmanager")
+    try:
+        response = secrets_manager.get_secret_value(SecretId=secret_arn)["SecretString"]
+    except Exception as e:
+        # pylint: disable=raise-missing-from
+        raise TaskCatException(
+            "ARN: {} encountered an error: {}".format(secret_arn, str(e))
+        )
+    return response
