@@ -5,8 +5,9 @@ from pathlib import Path
 from typing import Dict, Optional, Union
 
 import yaml
+from botocore.exceptions import ClientError
 
-from taskcat._cfn.template import Template
+from taskcat._cfn.template import Template, tcat_template_cache
 from taskcat._client_factory import Boto3Cache
 from taskcat._dataclasses import (
     BaseConfig,
@@ -42,9 +43,10 @@ DEFAULTS = {
 
 
 class Config:
-    def __init__(self, sources: list, uid: uuid.UUID):
+    def __init__(self, sources: list, uid: uuid.UUID, project_root: Path):
         self.config = BaseConfig.from_dict(DEFAULTS)
         self.config.set_source("TASKCAT_DEFAULT")
+        self.project_root = project_root
         self.uid = uid
         for source in sources:
             config_dict: dict = source["config"]
@@ -103,7 +105,7 @@ class Config:
         )
         if overrides_path.is_file():
             overrides = BaseConfig().to_dict()
-            with open(str(overrides_path), "r") as file_handle:
+            with open(str(overrides_path), "r", encoding="utf-8") as file_handle:
                 override_params = yaml.safe_load(file_handle)
             overrides["project"]["parameters"] = override_params
             sources.append({"source": str(overrides_path), "config": overrides})
@@ -119,9 +121,9 @@ class Config:
         # cli arguments
         if args:
             sources.append({"source": "CliArgument", "config": args})
-        return cls(sources=sources, uid=uid)
+        return cls(sources=sources, uid=uid, project_root=project_root)
 
-    # pylint: disable=protected-access
+    # pylint: disable=protected-access,inconsistent-return-statements
     @staticmethod
     def _get_project_source(base_cls, project_config_path, project_root, template_file):
         try:
@@ -129,7 +131,7 @@ class Config:
                 "source": str(project_config_path),
                 "config": base_cls._dict_from_file(project_config_path, fail_ok=False),
             }
-        except Exception as e:  # pylint: disable=broad-except
+        except FileNotFoundError as e:
             error = e
             try:
                 legacy_conf = parse_legacy_config(project_root)
@@ -137,9 +139,10 @@ class Config:
                     "source": str(project_root / "ci/taskcat.yml"),
                     "config": legacy_conf.to_dict(),
                 }
-            except Exception as e:  # pylint: disable=broad-except
+            except Exception as e:  # pylint: disable=broad-except,redefined-outer-name
                 LOG.debug(str(e), exc_info=True)
                 if not template_file:
+                    # pylint: disable=raise-missing-from
                     raise error
 
     @staticmethod
@@ -148,7 +151,7 @@ class Config:
         if not file_path.is_file() and fail_ok:
             return config_dict
         try:
-            with open(str(file_path), "r") as file_handle:
+            with open(str(file_path), "r", encoding="utf-8") as file_handle:
                 config_dict = yaml.safe_load(file_handle)
             return config_dict
         except Exception as e:  # pylint: disable=broad-except
@@ -171,7 +174,9 @@ class Config:
         if not file_path.is_file():
             raise TaskCatException(f"invalid template path {file_path}")
         try:
-            template = Template(str(file_path)).template
+            template = Template(
+                str(file_path), template_cache=tcat_template_cache
+            ).template
         except Exception as e:
             LOG.warning(f"failed to load template from {file_path}")
             LOG.debug(str(e), exc_info=True)
@@ -215,31 +220,49 @@ class Config:
                     config_dict[key_section][sub_key] = value
         return config_dict
 
-    def get_regions(self, boto3_cache: Boto3Cache = None):
+    def _get_regions(self, region_parameter_name, test, boto3_cache: Boto3Cache = None):
         if boto3_cache is None:
             boto3_cache = Boto3Cache()
+        region_object = {}
+        for region in getattr(test, region_parameter_name, []):
+            # TODO: comon_utils/determine_profile_for_region
+            profile = (
+                test.auth.get(region, test.auth.get("default", "default"))
+                if test.auth
+                else "default"
+            )
+            region_object[region] = RegionObj(
+                name=region,
+                account_id=boto3_cache.account_id(profile),
+                partition=boto3_cache.partition(profile),
+                profile=profile,
+                _boto3_cache=boto3_cache,
+                taskcat_id=self.uid,
+                _role_name=test.role_name,
+            )
+        return region_object
 
+    def get_regions(self, boto3_cache: Boto3Cache = None):
         region_objects: Dict[str, Dict[str, RegionObj]] = {}
         for test_name, test in self.config.tests.items():
-            region_objects[test_name] = {}
-            for region in test.regions:
-                profile = (
-                    test.auth.get(region, test.auth.get("default", "default"))
-                    if test.auth
-                    else "default"
+            region_objects[test_name] = self._get_regions("regions", test, boto3_cache)
+        return region_objects
+
+    def get_artifact_regions(self, boto3_cache: Boto3Cache = None):
+        region_objects: Dict[str, Dict[str, RegionObj]] = {}
+        for test_name, test in self.config.tests.items():
+            if test.artifact_regions is not None:
+                region_objects[test_name] = self._get_regions(
+                    "artifact_regions", test, boto3_cache
                 )
-                region_objects[test_name][region] = RegionObj(
-                    name=region,
-                    account_id=boto3_cache.account_id(profile),
-                    partition=boto3_cache.partition(profile),
-                    profile=profile,
-                    _boto3_cache=boto3_cache,
-                    taskcat_id=self.uid,
+            else:
+                region_objects[test_name] = self._get_regions(
+                    "regions", test, boto3_cache
                 )
         return region_objects
 
     def get_buckets(self, boto3_cache: Boto3Cache = None):
-        regions = self.get_regions(boto3_cache)
+        regions = self.get_artifact_regions(boto3_cache)
         bucket_objects: Dict[str, S3BucketObj] = {}
         bucket_mappings: Dict[str, Dict[str, S3BucketObj]] = {}
         for test_name, test in self.config.tests.items():
@@ -267,6 +290,7 @@ class Config:
             else "private"
         )
         sigv4 = not self.config.project.s3_enable_sig_v2
+        org_id = self.config.project.org_id
         if not test.s3_bucket and not bucket_objects.get(region.account_id):
             name = generate_bucket_name(self.config.project.name)
             auto_generated = True
@@ -289,6 +313,7 @@ class Config:
             taskcat_id=self.uid,
             partition=region.partition,
             regional_buckets=test.s3_regional_buckets,
+            org_id=org_id,
         )
         if new:
             bucket_obj.create()
@@ -302,7 +327,11 @@ class Config:
             if self.config.project.s3_object_acl
             else "private"
         )
+
         sigv4 = not self.config.project.s3_enable_sig_v2
+
+        org_id = self.config.project.org_id
+
         if not test.s3_bucket and not bucket_objects.get(_bucket_obj_key):
             name = generate_regional_bucket_name(region)
             auto_generated = True
@@ -311,8 +340,15 @@ class Config:
             name = bucket_objects[_bucket_obj_key].name
             auto_generated = bucket_objects[_bucket_obj_key].auto_generated
         else:
-            name = test.s3_bucket
+            name = f"{test.s3_bucket}-{region.name}"
             auto_generated = False
+            try:
+                region.client("s3").head_bucket(Bucket=name)
+            except ClientError as e:
+                if "(404)" in str(e):
+                    new = True
+                else:
+                    raise
         bucket_obj = S3BucketObj(
             name=name,
             region=region.name,
@@ -324,6 +360,7 @@ class Config:
             taskcat_id=self.uid,
             partition=region.partition,
             regional_buckets=test.s3_regional_buckets,
+            org_id=org_id,
         )
         if new:
             bucket_obj.create()
@@ -351,7 +388,14 @@ class Config:
                 region = region_objects[test_name][region_name]
                 s3bucket = bucket_objects[test_name][region_name]
                 parameters[test_name][region_name] = ParamGen(
-                    region_params, s3bucket.name, region.name, region.client
+                    self.project_root,
+                    region_params,
+                    s3bucket.name,
+                    region.name,
+                    region.client,
+                    self.config.project.name,
+                    test_name,
+                    test.az_blacklist,
                 ).results
         return parameters
 
@@ -362,20 +406,22 @@ class Config:
             parameters[test_name] = template.parameters()
         return parameters
 
-    def get_templates(self, project_root: Path):
+    def get_templates(self):
         templates = {}
         for test_name, test in self.config.tests.items():
             templates[test_name] = Template(
-                template_path=project_root / test.template,
-                project_root=project_root,
+                template_path=self.project_root / test.template,
+                project_root=self.project_root,
                 s3_key_prefix=f"{self.config.project.name}/",
+                template_cache=tcat_template_cache,
             )
         return templates
 
-    def get_tests(self, project_root, templates, regions, buckets, parameters):
+    def get_tests(self, templates, regions, buckets, parameters):
         tests = {}
         for test_name, test in self.config.tests.items():
             region_list = []
+            artifact_region_list = []
             tag_list = []
             if test.tags:
                 for tag_key, tag_value in test.tags.items():
@@ -388,12 +434,20 @@ class Config:
                         parameters[test_name][region_obj.name],
                     )
                 )
+
             tests[test_name] = TestObj(
                 name=test_name,
-                template_path=project_root / test.template,
+                template_path=self.project_root / test.template,
                 template=templates[test_name],
-                project_root=project_root,
+                project_root=self.project_root,
                 regions=region_list,
+                artifact_regions=artifact_region_list,
                 tags=tag_list,
+                uid=self.uid,
+                _project_name=self.config.project.name,
+                _shorten_stack_name=self.config.project.shorten_stack_name,
+                _stack_name=test.stack_name,
+                _stack_name_prefix=test.stack_name_prefix,
+                _stack_name_suffix=test.stack_name_suffix,
             )
         return tests
