@@ -4,13 +4,21 @@ import re
 import uuid
 from typing import Set
 
-from taskcat._common_utils import CommonTools
+import dulwich
+from dulwich.repo import Repo
+from taskcat._common_utils import (
+    CommonTools,
+    fetch_secretsmanager_parameter_value,
+    fetch_ssm_parameter_value,
+)
 from taskcat.exceptions import TaskCatException
 
 LOG = logging.getLogger(__name__)
 
 
+# pylint: disable=too-many-instance-attributes
 class ParamGen:
+    RE_GITBRANCH = re.compile(r"\$\[taskcat_git_branch\]", re.IGNORECASE)
     RE_GETURL = re.compile(r"(?<=._url_)(.+)(?=]$)", re.IGNORECASE)
     RE_COUNT = re.compile(r"(?!\w+_)\d{1,2}", re.IGNORECASE)
     RE_PWTYPE = re.compile(r"(?<=_genpass_)((\d+)(\w)(\]))", re.IGNORECASE)
@@ -18,6 +26,9 @@ class ParamGen:
     RE_GENRANDSTR = re.compile(r"\$\[taskcat_random-string]", re.IGNORECASE)
     RE_GENNUMB = re.compile(r"\$\[taskcat_random-numbers]", re.IGNORECASE)
     RE_GENAUTOBUCKET = re.compile(r"\$\[taskcat_autobucket]", re.IGNORECASE)
+    RE_GENAUTOBUCKETPREFIX = re.compile(
+        r"\$\[taskcat_autobucket_prefix]", re.IGNORECASE
+    )
     RE_GENAZ = re.compile(r"\$\[\w+_ge[nt]az_\d]", re.IGNORECASE)
     RE_GENAZ_SINGLE = re.compile(
         r"\$\[\w+_ge[nt]singleaz_(?P<az_id>\d+)]", re.IGNORECASE
@@ -32,8 +43,24 @@ class ParamGen:
     )
     RE_GETVAL = re.compile(r"(?<=._getval_)(\w+)(?=]$)", re.IGNORECASE)
     RE_CURRENT_REGION = re.compile(r"\$\[taskcat_current_region]", re.IGNORECASE)
+    RE_PROJECT_NAME = re.compile(r"\$\[taskcat_project_name]", re.IGNORECASE)
+    RE_TEST_NAME = re.compile(r"\$\[taskcat_test_name]", re.IGNORECASE)
+    RE_SSM_PARAMETER = re.compile(r"\$\[taskcat_ssm_.*]$", re.IGNORECASE)
+    RE_SECRETSMANAGER_PARAMETER = re.compile(
+        r"\$\[taskcat_secretsmanager_.*]$", re.IGNORECASE
+    )
 
-    def __init__(self, param_dict, bucket_name, region, boto_client, az_excludes=None):
+    def __init__(
+        self,
+        project_root,
+        param_dict,
+        bucket_name,
+        region,
+        boto_client,
+        project_name,
+        test_name,
+        az_excludes=None,
+    ):
         self.regxfind = CommonTools.regxfind
         self._param_dict = param_dict
         _missing_params = []
@@ -55,6 +82,9 @@ class ParamGen:
         self.bucket_name = bucket_name
         self._boto_client = boto_client
         self.region = region
+        self.project_name = project_name
+        self.test_name = test_name
+        self.project_root = project_root
         if not az_excludes:
             self.az_excludes: Set[str] = set()
         else:
@@ -72,13 +102,16 @@ class ParamGen:
                 for idx, value in enumerate(param_value):
                     _nested_param_dict[idx] = value
                 nested_pg = ParamGen(
+                    self.project_root,
                     _nested_param_dict,
                     self.bucket_name,
                     self.region,
                     self._boto_client,
+                    self.project_name,
+                    self.test_name,
                     self.az_excludes,
                 )
-                nested_pg.transform_parameter()
+                # nested_pg.transform_parameter()
                 for result_value in nested_pg.results.values():
                     _results_list.append(result_value)
                 self.param_value = _results_list
@@ -102,6 +135,10 @@ class ParamGen:
             self._regex_replace_param_value(
                 self.RE_GENAUTOBUCKET, self._gen_autobucket()
             )
+            # $[taskcat_autobucket_prefix]
+            self._regex_replace_param_value(
+                self.RE_GENAUTOBUCKETPREFIX, self._get_autobucket_prefix()
+            )
 
             # $[taskcat_genpass_X]
             self._gen_password_wrapper(self.RE_GENPW, self.RE_PWTYPE, self.RE_COUNT)
@@ -133,10 +170,19 @@ class ParamGen:
             # $[taskcat_genuuid]
             self._regex_replace_param_value(self.RE_GENUUID, self._gen_uuid())
 
+            # $[taskcat_ssm_X]
+            self._get_ssm_param_value_wrapper(self.RE_SSM_PARAMETER)
             # $[taskcat_current_region]
             self._regex_replace_param_value(
                 self.RE_CURRENT_REGION, self._gen_current_region()
             )
+            self._regex_replace_param_value(
+                self.RE_PROJECT_NAME, self._get_project_name()
+            )
+            self._regex_replace_param_value(self.RE_TEST_NAME, self._get_test_name())
+            # $[taskcat_git_branch]
+            self._git_branch_wrapper(self.RE_GITBRANCH)
+
             self.results.update({self.param_name: self.param_value})
 
     def get_available_azs(self, count):
@@ -183,7 +229,7 @@ class ParamGen:
         """
 
         regional_azs = self.get_available_azs(az_id)
-        return regional_azs.split(",")[-1]
+        return regional_azs.split(",")[-1]  # pylint: disable=use-maxsplit-arg
 
     def get_content(self, bucket, object_key):
         """
@@ -287,9 +333,32 @@ class ParamGen:
     def _gen_autobucket(self):
         return self.bucket_name
 
+    def _get_autobucket_prefix(self):
+        bucket_prefix = self.bucket_name
+        if bucket_prefix.lower().startswith("tcat"):
+            bucket_prefix = "-".join(bucket_prefix.split("-")[:2])
+        return bucket_prefix
+
     def _gen_current_region(self):
         return self.region
 
+    def _get_project_name(self):
+        return self.project_name
+
+    def _get_test_name(self):
+        return self.test_name
+
+    def _get_git_branch(self):
+        try:
+            repo = Repo(self.project_root)
+        except dulwich.errors.NotGitRepository as exc:
+            raise TaskCatException(
+                f"Project root: ({self.project_root}) is not a git repository. Cannot use '$[taskcat_git_branch]' psuedo-parameter"
+            ) from exc
+        (_, ref), _ = repo.refs.follow(b"HEAD")
+        match = re.search(r"/([^/]+)$", ref.decode("utf-8"))
+        return match[1]
+
     def _gen_password_wrapper(self, gen_regex, type_regex, count_regex):
         if gen_regex.search(self.param_value):
             passlen = int(self.regxfind(count_regex, self.param_value))
@@ -312,6 +381,10 @@ class ParamGen:
                 param_value = self.genpassword(passlen, gentype)
                 self._regex_replace_param_value(gen_regex, param_value)
 
+    def _git_branch_wrapper(self, git_branch_regex):
+        if git_branch_regex.search(self.param_value):
+            self._regex_replace_param_value(git_branch_regex, self._get_git_branch())
+
     def _gen_az_wrapper(self, genaz_regex, count_regex):
         if genaz_regex.search(self.param_value):
             numazs = int(self.regxfind(count_regex, self.param_value))
@@ -365,6 +438,24 @@ class ParamGen:
             self._regex_replace_param_value(re.compile("^.*$"), param_value)
             self._regex_replace_param_value(re.compile("^.*$"), param_value)
 
+    def _get_ssm_param_value_wrapper(self, ssm_param_value_regex):
+        if ssm_param_value_regex.search(self.param_value):
+            ssm_value_str = self.regxfind(ssm_param_value_regex, self.param_value)
+            param_path = "_".join(ssm_value_str[:-1].split("_")[2:])
+            param_value = fetch_ssm_parameter_value(self._boto_client, param_path)
+            self._regex_replace_param_value(re.compile("^.*"), param_value)
+
+    def _get_secretsmanager_param_value_wrapper(self, secretsmanager_param_value_regex):
+        if secretsmanager_param_value_regex.search(self.param_value):
+            sm_value_str = self.regxfind(
+                secretsmanager_param_value_regex, self.param_value
+            )
+            sm_arn = "_".join(sm_value_str.split("_")[2:])
+            param_value = fetch_secretsmanager_parameter_value(
+                self._boto_client, sm_arn
+            )
+            self._regex_replace_param_value(re.compile("^.*"), param_value)
+
     def _getval_wrapper(self, getval_regex):
         if getval_regex.search(self.param_value):
             requested_key = self.regxfind(getval_regex, self.param_value)
